@@ -5,10 +5,8 @@
 #ifndef MEDIAENGINEDEFAULT_H_
 #define MEDIAENGINEDEFAULT_H_
 
-#include "nsINamed.h"
 #include "nsITimer.h"
 
-#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "DOMMediaStream.h"
 #include "nsComponentManagerUtils.h"
@@ -19,9 +17,8 @@
 #include "MediaEnginePrefs.h"
 #include "VideoSegment.h"
 #include "AudioSegment.h"
-#include "StreamTracks.h"
 #include "MediaEngineSource.h"
-#include "MediaStreamGraph.h"
+#include "MediaTrackGraph.h"
 
 namespace mozilla {
 
@@ -43,22 +40,21 @@ class MediaEngineDefaultVideoSource : public MediaEngineSource {
   nsString GetGroupId() const override;
 
   nsresult Allocate(const dom::MediaTrackConstraints& aConstraints,
-                    const MediaEnginePrefs& aPrefs, const nsString& aDeviceId,
-                    const ipc::PrincipalInfo& aPrincipalInfo,
+                    const MediaEnginePrefs& aPrefs, uint64_t aWindowID,
                     const char** aOutBadConstraint) override;
-  void SetTrack(const RefPtr<SourceMediaStream>& aStream, TrackID aTrackID,
+  void SetTrack(const RefPtr<SourceMediaTrack>& aTrack,
                 const PrincipalHandle& aPrincipal) override;
   nsresult Start() override;
   nsresult Reconfigure(const dom::MediaTrackConstraints& aConstraints,
                        const MediaEnginePrefs& aPrefs,
-                       const nsString& aDeviceId,
                        const char** aOutBadConstraint) override;
   nsresult Stop() override;
   nsresult Deallocate() override;
 
   uint32_t GetBestFitnessDistance(
-      const nsTArray<const NormalizedConstraintSet*>& aConstraintSets,
-      const nsString& aDeviceId) const override;
+      const nsTArray<const NormalizedConstraintSet*>& aConstraintSets)
+      const override;
+  void GetSettings(dom::MediaTrackSettings& aOutSettings) const override;
 
   bool IsFake() const override { return true; }
 
@@ -81,14 +77,16 @@ class MediaEngineDefaultVideoSource : public MediaEngineSource {
   // Current state of this source.
   MediaEngineSourceState mState = kReleased;
   RefPtr<layers::Image> mImage;
-  RefPtr<SourceMediaStream> mStream;
-  TrackID mTrackID = TRACK_NONE;
+  RefPtr<SourceMediaTrack> mTrack;
   PrincipalHandle mPrincipalHandle = PRINCIPAL_HANDLE_NONE;
 
   MediaEnginePrefs mOpts;
   int mCb = 16;
   int mCr = 16;
 
+  // Main thread only.
+  const RefPtr<media::Refcountable<dom::MediaTrackSettings>> mSettings;
+
  private:
   const nsString mName;
 };
@@ -104,15 +102,13 @@ class MediaEngineDefaultAudioSource : public MediaEngineSource {
   nsString GetGroupId() const override;
 
   nsresult Allocate(const dom::MediaTrackConstraints& aConstraints,
-                    const MediaEnginePrefs& aPrefs, const nsString& aDeviceId,
-                    const ipc::PrincipalInfo& aPrincipalInfo,
+                    const MediaEnginePrefs& aPrefs, uint64_t aWindowID,
                     const char** aOutBadConstraint) override;
-  void SetTrack(const RefPtr<SourceMediaStream>& aStream, TrackID aTrackID,
+  void SetTrack(const RefPtr<SourceMediaTrack>& aTrack,
                 const PrincipalHandle& aPrincipal) override;
   nsresult Start() override;
   nsresult Reconfigure(const dom::MediaTrackConstraints& aConstraints,
                        const MediaEnginePrefs& aPrefs,
-                       const nsString& aDeviceId,
                        const char** aOutBadConstraint) override;
   nsresult Stop() override;
   nsresult Deallocate() override;
@@ -123,17 +119,14 @@ class MediaEngineDefaultAudioSource : public MediaEngineSource {
     return dom::MediaSourceEnum::Microphone;
   }
 
-  uint32_t GetBestFitnessDistance(
-      const nsTArray<const NormalizedConstraintSet*>& aConstraintSets,
-      const nsString& aDeviceId) const override;
+  void GetSettings(dom::MediaTrackSettings& aOutSettings) const override;
 
  protected:
   ~MediaEngineDefaultAudioSource();
 
   // Current state of this source.
   MediaEngineSourceState mState = kReleased;
-  RefPtr<SourceMediaStream> mStream;
-  TrackID mTrackID = TRACK_NONE;
+  RefPtr<SourceMediaTrack> mTrack;
   PrincipalHandle mPrincipalHandle = PRINCIPAL_HANDLE_NONE;
   uint32_t mFrequency = 1000;
   RefPtr<AudioSourcePullListener> mPullListener;
@@ -147,8 +140,13 @@ class MediaEngineDefault : public MediaEngine {
                         nsTArray<RefPtr<MediaDevice>>*) override;
   void Shutdown() override {}
 
+  MediaEventSource<void>& DeviceListChangeEvent() override {
+    return mDeviceListChangeEvent;
+  }
+
  private:
   ~MediaEngineDefault() = default;
+  MediaEventProducer<void> mDeviceListChangeEvent;
 };
 
 }  // namespace mozilla
