@@ -10,7 +10,6 @@
 #include "LocalStorageCache.h"
 #include "LocalStorageManager.h"
 
-#include "nsIEffectiveTLDService.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsThreadUtils.h"
@@ -18,7 +17,6 @@
 #include "mozStorageCID.h"
 #include "mozStorageHelper.h"
 #include "mozIStorageService.h"
-#include "mozIStorageBindingParamsArray.h"
 #include "mozIStorageBindingParams.h"
 #include "mozIStorageValueArray.h"
 #include "mozIStorageFunction.h"
@@ -141,14 +139,14 @@ StorageDBThread* StorageDBThread::GetOrCreate(const nsString& aProfilePath) {
     return sStorageThread;
   }
 
-  nsAutoPtr<StorageDBThread> storageThread(new StorageDBThread());
+  auto storageThread = MakeUnique<StorageDBThread>();
 
   nsresult rv = storageThread->Init(aProfilePath);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return nullptr;
   }
 
-  sStorageThread = storageThread.forget();
+  sStorageThread = storageThread.release();
 
   return sStorageThread;
 }
@@ -313,8 +311,7 @@ bool StorageDBThread::ShouldPreloadOrigin(const nsACString& aOrigin) {
   return mOriginsHavingData.Contains(aOrigin);
 }
 
-void StorageDBThread::GetOriginsHavingData(
-    InfallibleTArray<nsCString>* aOrigins) {
+void StorageDBThread::GetOriginsHavingData(nsTArray<nsCString>* aOrigins) {
   MonitorAutoLock monitor(mThreadObserver->GetMonitor());
   for (auto iter = mOriginsHavingData.Iter(); !iter.Done(); iter.Next()) {
     aOrigins->AppendElement(iter.Get()->GetKey());
@@ -325,7 +322,7 @@ nsresult StorageDBThread::InsertDBOp(StorageDBThread::DBOperation* aOperation) {
   MonitorAutoLock monitor(mThreadObserver->GetMonitor());
 
   // Sentinel to don't forget to delete the operation when we exit early.
-  nsAutoPtr<StorageDBThread::DBOperation> opScope(aOperation);
+  UniquePtr<StorageDBThread::DBOperation> opScope(aOperation);
 
   if (NS_FAILED(mStatus)) {
     MonitorAutoUnlock unlock(mThreadObserver->GetMonitor());
@@ -363,7 +360,7 @@ nsresult StorageDBThread::InsertDBOp(StorageDBThread::DBOperation* aOperation) {
         aOperation->Finalize(NS_OK);
         return NS_OK;
       }
-      MOZ_FALLTHROUGH;
+      [[fallthrough]];
 
     case DBOperation::opGetUsage:
       if (aOperation->Type() == DBOperation::opPreloadUrgent) {
@@ -374,7 +371,7 @@ nsresult StorageDBThread::InsertDBOp(StorageDBThread::DBOperation* aOperation) {
       }
 
       // DB operation adopted, don't delete it.
-      opScope.forget();
+      Unused << opScope.release();
 
       // Immediately start executing this.
       monitor.Notify();
@@ -386,7 +383,7 @@ nsresult StorageDBThread::InsertDBOp(StorageDBThread::DBOperation* aOperation) {
       mPendingTasks.Add(aOperation);
 
       // DB operation adopted, don't delete it.
-      opScope.forget();
+      Unused << opScope.release();
 
       ScheduleFlush();
       break;
@@ -471,7 +468,7 @@ void StorageDBThread::ThreadFunc() {
       }
       NotifyFlushCompletion();
     } else if (MOZ_LIKELY(mPreloads.Length())) {
-      nsAutoPtr<DBOperation> op(mPreloads[0]);
+      UniquePtr<DBOperation> op(mPreloads[0]);
       mPreloads.RemoveElementAt(0);
       {
         MonitorAutoUnlock unlockMonitor(mThreadObserver->GetMonitor());
@@ -483,7 +480,6 @@ void StorageDBThread::ThreadFunc() {
       }
     } else if (MOZ_UNLIKELY(!mStopIOThread)) {
       AUTO_PROFILER_LABEL("StorageDBThread::ThreadFunc::Wait", IDLE);
-      AUTO_PROFILER_THREAD_SLEEP;
       lockMonitor.Wait(timeUntilFlush);
     }
   }  // thread loop
@@ -799,7 +795,7 @@ class OriginAttrsPatternMatchSQLFunction final : public mozIStorageFunction {
 
  private:
   OriginAttrsPatternMatchSQLFunction() = delete;
-  ~OriginAttrsPatternMatchSQLFunction() {}
+  ~OriginAttrsPatternMatchSQLFunction() = default;
 
   OriginAttributesPattern mPattern;
 };
@@ -1295,7 +1291,7 @@ void StorageDBThread::PendingOperations::Add(
       // logic, if we would not delete the update tasks, changes would have been
       // stored to the database after clear operations have been executed.
       for (auto iter = mUpdates.Iter(); !iter.Done(); iter.Next()) {
-        nsAutoPtr<DBOperation>& pendingTask = iter.Data();
+        const auto& pendingTask = iter.Data();
 
         if (aOperation->Type() == DBOperation::opClear &&
             (pendingTask->OriginNoSuffix() != aOperation->OriginNoSuffix() ||
@@ -1344,12 +1340,12 @@ bool StorageDBThread::PendingOperations::Prepare() {
   // all scope-related update operations we have here now were
   // scheduled after the clear operations.
   for (auto iter = mClears.Iter(); !iter.Done(); iter.Next()) {
-    mExecList.AppendElement(iter.Data().forget());
+    mExecList.AppendElement(std::move(iter.Data()));
   }
   mClears.Clear();
 
   for (auto iter = mUpdates.Iter(); !iter.Done(); iter.Next()) {
-    mExecList.AppendElement(iter.Data().forget());
+    mExecList.AppendElement(std::move(iter.Data()));
   }
   mUpdates.Clear();
 
@@ -1364,7 +1360,7 @@ nsresult StorageDBThread::PendingOperations::Execute(StorageDBThread* aThread) {
   nsresult rv;
 
   for (uint32_t i = 0; i < mExecList.Length(); ++i) {
-    StorageDBThread::DBOperation* task = mExecList[i];
+    const auto& task = mExecList[i];
     rv = task->Perform(aThread);
     if (NS_FAILED(rv)) {
       return rv;
@@ -1447,7 +1443,7 @@ bool StorageDBThread::PendingOperations::IsOriginClearPending(
 
   for (uint32_t i = 0; i < mExecList.Length(); ++i) {
     if (FindPendingClearForOrigin(aOriginSuffix, aOriginNoSuffix,
-                                  mExecList[i])) {
+                                  mExecList[i].get())) {
       return true;
     }
   }
@@ -1488,7 +1484,7 @@ bool StorageDBThread::PendingOperations::IsOriginUpdatePending(
 
   for (uint32_t i = 0; i < mExecList.Length(); ++i) {
     if (FindPendingUpdateForOrigin(aOriginSuffix, aOriginNoSuffix,
-                                   mExecList[i])) {
+                                   mExecList[i].get())) {
       return true;
     }
   }
