@@ -1,14 +1,12 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "txMozillaXMLOutput.h"
 
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIDocShell.h"
-#include "nsIDOMDocument.h"
-#include "nsIDOMDocumentType.h"
 #include "nsIScriptElement.h"
 #include "nsCharsetSource.h"
 #include "nsIRefreshURI.h"
@@ -18,20 +16,19 @@
 #include "nsUnicharUtils.h"
 #include "nsGkAtoms.h"
 #include "txLog.h"
-#include "nsIConsoleService.h"
-#include "nsIDOMDocumentFragment.h"
 #include "nsNameSpaceManager.h"
 #include "txStringUtils.h"
 #include "txURIUtils.h"
-#include "nsIHTMLDocument.h"
-#include "nsIStyleSheetLinkingElement.h"
 #include "nsIDocumentTransformer.h"
 #include "mozilla/StyleSheetInlines.h"
 #include "mozilla/css/Loader.h"
+#include "mozilla/dom/DocumentType.h"
+#include "mozilla/dom/DocumentFragment.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/ScriptLoader.h"
 #include "mozilla/Encoding.h"
 #include "nsContentUtils.h"
+#include "nsDocElementCreatedNotificationRunner.h"
 #include "txXMLUtils.h"
 #include "nsContentSink.h"
 #include "nsINode.h"
@@ -72,7 +69,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
 }
 
 txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
-                                       nsIDOMDocumentFragment* aFragment,
+                                       DocumentFragment* aFragment,
                                        bool aNoFixup)
     : mTreeDepth(0),
       mBadChildLevel(0),
@@ -85,7 +82,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
   mOutputFormat.merge(*aFormat);
   mOutputFormat.setFromDefaults();
 
-  mCurrentNode = do_QueryInterface(aFragment);
+  mCurrentNode = aFragment;
   mDocument = mCurrentNode->OwnerDoc();
   mNodeInfoManager = mDocument->NodeInfoManager();
 }
@@ -177,7 +174,7 @@ nsresult txMozillaXMLOutput::comment(const nsString& aData) {
 
   TX_ENSURE_CURRENTNODE;
 
-  RefPtr<Comment> comment = new Comment(mNodeInfoManager);
+  RefPtr<Comment> comment = new (mNodeInfoManager) Comment(mNodeInfoManager);
 
   rv = comment->SetText(aData, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -206,13 +203,11 @@ nsresult txMozillaXMLOutput::endDocument(nsresult aResult) {
   }
 
   if (mCreatingNewDocument) {
-    // This should really be handled by nsIDocument::EndLoad
-    MOZ_ASSERT(
-        mDocument->GetReadyStateEnum() == nsIDocument::READYSTATE_LOADING,
-        "Bad readyState");
-    mDocument->SetReadyStateInternal(nsIDocument::READYSTATE_INTERACTIVE);
-    ScriptLoader* loader = mDocument->ScriptLoader();
-    if (loader) {
+    // This should really be handled by Document::EndLoad
+    MOZ_ASSERT(mDocument->GetReadyStateEnum() == Document::READYSTATE_LOADING,
+               "Bad readyState");
+    mDocument->SetReadyStateInternal(Document::READYSTATE_INTERACTIVE);
+    if (ScriptLoader* loader = mDocument->ScriptLoader()) {
       loader->ParsingComplete(false);
     }
   }
@@ -222,9 +217,9 @@ nsresult txMozillaXMLOutput::endDocument(nsresult aResult) {
     if (win) {
       nsCOMPtr<nsIRefreshURI> refURI = do_QueryInterface(win->GetDocShell());
       if (refURI) {
-        refURI->SetupRefreshURIFromHeader(mDocument->GetDocBaseURI(),
-                                          mDocument->NodePrincipal(),
-                                          mRefreshString);
+        refURI->SetupRefreshURIFromHeader(
+            mDocument->GetDocBaseURI(), mDocument->NodePrincipal(),
+            mDocument->InnerWindowID(), mRefreshString);
       }
     }
   }
@@ -264,10 +259,10 @@ nsresult txMozillaXMLOutput::endElement() {
     }
 
     // Handle elements that are different when parser-created
-    if (element->IsAnyOfHTMLElements(nsGkAtoms::title, nsGkAtoms::object,
-                                     nsGkAtoms::select, nsGkAtoms::textarea) ||
-        element->IsSVGElement(nsGkAtoms::title)) {
-      element->DoneAddingChildren(true);
+    if (nsIContent::RequiresDoneCreatingElement(
+            element->NodeInfo()->NamespaceID(),
+            element->NodeInfo()->NameAtom())) {
+      element->DoneCreatingElement();
     } else if (element->IsSVGElement(nsGkAtoms::script) ||
                element->IsHTMLElement(nsGkAtoms::script)) {
       nsCOMPtr<nsIScriptElement> sele = do_QueryInterface(element);
@@ -284,24 +279,20 @@ nsresult txMozillaXMLOutput::endElement() {
                    "Script elements need to implement nsIScriptElement and SVG "
                    "wasn't disabled.");
       }
-    } else if (element->IsAnyOfHTMLElements(
-                   nsGkAtoms::input, nsGkAtoms::button, nsGkAtoms::menuitem,
-                   nsGkAtoms::audio, nsGkAtoms::video)) {
-      element->DoneCreatingElement();
+    } else if (nsIContent::RequiresDoneAddingChildren(
+                   element->NodeInfo()->NamespaceID(),
+                   element->NodeInfo()->NameAtom())) {
+      element->DoneAddingChildren(true);
     }
   }
 
   if (mCreatingNewDocument) {
     // Handle all sorts of stylesheets
-    nsCOMPtr<nsIStyleSheetLinkingElement> ssle =
-        do_QueryInterface(mCurrentNode);
-    if (ssle) {
-      ssle->SetEnableUpdates(true);
-      bool willNotify;
-      bool isAlternate;
-      nsresult rv =
-          ssle->UpdateStyleSheet(mNotifier, &willNotify, &isAlternate);
-      if (mNotifier && NS_SUCCEEDED(rv) && willNotify && !isAlternate) {
+    if (auto* linkStyle = LinkStyle::FromNode(*mCurrentNode)) {
+      linkStyle->SetEnableUpdates(true);
+      auto updateOrError = linkStyle->UpdateStyleSheet(mNotifier);
+      if (mNotifier && updateOrError.isOk() &&
+          updateOrError.unwrap().ShouldBlock()) {
         mNotifier->AddPendingStylesheet();
       }
     }
@@ -339,8 +330,8 @@ nsresult txMozillaXMLOutput::endElement() {
   return NS_OK;
 }
 
-void txMozillaXMLOutput::getOutputDocument(nsIDOMDocument** aDocument) {
-  CallQueryInterface(mDocument, aDocument);
+void txMozillaXMLOutput::getOutputDocument(Document** aDocument) {
+  NS_IF_ADDREF(*aDocument = mDocument);
 }
 
 nsresult txMozillaXMLOutput::processingInstruction(const nsString& aTarget,
@@ -358,24 +349,22 @@ nsresult txMozillaXMLOutput::processingInstruction(const nsString& aTarget,
   nsCOMPtr<nsIContent> pi =
       NS_NewXMLProcessingInstruction(mNodeInfoManager, aTarget, aData);
 
-  nsCOMPtr<nsIStyleSheetLinkingElement> ssle;
+  LinkStyle* linkStyle = nullptr;
   if (mCreatingNewDocument) {
-    ssle = do_QueryInterface(pi);
-    if (ssle) {
-      ssle->InitStyleLinkElement(false);
-      ssle->SetEnableUpdates(false);
+    linkStyle = LinkStyle::FromNode(*pi);
+    if (linkStyle) {
+      linkStyle->SetEnableUpdates(false);
     }
   }
 
   rv = mCurrentNode->AppendChildTo(pi, true);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (ssle) {
-    ssle->SetEnableUpdates(true);
-    bool willNotify;
-    bool isAlternate;
-    rv = ssle->UpdateStyleSheet(mNotifier, &willNotify, &isAlternate);
-    if (mNotifier && NS_SUCCEEDED(rv) && willNotify && !isAlternate) {
+  if (linkStyle) {
+    linkStyle->SetEnableUpdates(true);
+    auto updateOrError = linkStyle->UpdateStyleSheet(mNotifier);
+    if (mNotifier && updateOrError.isOk() &&
+        updateOrError.unwrap().ShouldBlock()) {
       mNotifier->AddPendingStylesheet();
     }
   }
@@ -401,8 +390,8 @@ nsresult txMozillaXMLOutput::startDocument() {
 nsresult txMozillaXMLOutput::startElement(nsAtom* aPrefix, nsAtom* aLocalName,
                                           nsAtom* aLowercaseLocalName,
                                           const int32_t aNsID) {
-  NS_PRECONDITION(aNsID != kNameSpaceID_None || !aPrefix,
-                  "Can't have prefix without namespace");
+  MOZ_ASSERT(aNsID != kNameSpaceID_None || !aPrefix,
+             "Can't have prefix without namespace");
 
   if (mOutputFormat.mMethod == eHTMLOutput && aNsID == kNameSpaceID_None) {
     RefPtr<nsAtom> owner;
@@ -505,11 +494,8 @@ nsresult txMozillaXMLOutput::startElementInternal(nsAtom* aPrefix,
 
   if (mCreatingNewDocument) {
     // Handle all sorts of stylesheets
-    nsCOMPtr<nsIStyleSheetLinkingElement> ssle =
-        do_QueryInterface(mOpenedElement);
-    if (ssle) {
-      ssle->InitStyleLinkElement(false);
-      ssle->SetEnableUpdates(false);
+    if (auto* linkStyle = LinkStyle::FromNodeOrNull(mOpenedElement)) {
+      linkStyle->SetEnableUpdates(false);
     }
   }
 
@@ -536,7 +522,8 @@ nsresult txMozillaXMLOutput::closePrevious(bool aFlushText) {
 
     if (currentIsDoc) {
       mRootContentCreated = true;
-      nsContentSink::NotifyDocElementCreated(mDocument);
+      nsContentUtils::AddScriptRunner(
+          new nsDocElementCreatedNotificationRunner(mDocument));
     }
 
     mCurrentNode = mOpenedElement;
@@ -553,7 +540,8 @@ nsresult txMozillaXMLOutput::closePrevious(bool aFlushText) {
       rv = createTxWrapper();
       NS_ENSURE_SUCCESS(rv, rv);
     }
-    RefPtr<nsTextNode> text = new nsTextNode(mNodeInfoManager);
+    RefPtr<nsTextNode> text =
+        new (mNodeInfoManager) nsTextNode(mNodeInfoManager);
 
     rv = text->SetText(mText, false);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -726,7 +714,7 @@ void txMozillaXMLOutput::processHTTPEquiv(nsAtom* aHeader,
 
 nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
                                                   int32_t aNsID,
-                                                  nsIDocument* aSourceDocument,
+                                                  Document* aSourceDocument,
                                                   bool aLoadedAsData) {
   nsresult rv;
 
@@ -740,11 +728,11 @@ nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
     rv = NS_NewXMLDocument(getter_AddRefs(mDocument), aLoadedAsData);
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  // This should really be handled by nsIDocument::BeginLoad
+  // This should really be handled by Document::BeginLoad
   MOZ_ASSERT(
-      mDocument->GetReadyStateEnum() == nsIDocument::READYSTATE_UNINITIALIZED,
+      mDocument->GetReadyStateEnum() == Document::READYSTATE_UNINITIALIZED,
       "Bad readyState");
-  mDocument->SetReadyStateInternal(nsIDocument::READYSTATE_LOADING);
+  mDocument->SetReadyStateInternal(Document::READYSTATE_LOADING);
   mDocument->SetMayStartLayout(false);
   bool hasHadScriptObject = false;
   nsIScriptGlobalObject* sgo =
@@ -761,6 +749,8 @@ nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
   // source, so that we have the right principal.
   mDocument->SetScriptHandlingObject(sgo);
 
+  mDocument->SetStateObjectFrom(aSourceDocument);
+
   // Set the charset
   if (!mOutputFormat.mEncoding.IsEmpty()) {
     const Encoding* encoding = Encoding::ForLabel(mOutputFormat.mEncoding);
@@ -813,9 +803,8 @@ nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
 
   // Do this after calling OnDocumentCreated to ensure that the
   // PresShell/PresContext has been hooked up and get notified.
-  nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(mDocument);
-  if (htmlDoc) {
-    htmlDoc->SetCompatibilityMode(eCompatibility_FullStandards);
+  if (mDocument) {
+    mDocument->SetCompatibilityMode(eCompatibility_FullStandards);
   }
 
   // Add a doc-type if requested
@@ -827,8 +816,6 @@ nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
       qName.Assign(aName);
     }
 
-    nsCOMPtr<nsIDOMDocumentType> documentType;
-
     nsresult rv = nsContentUtils::CheckQName(qName);
     if (NS_SUCCEEDED(rv)) {
       RefPtr<nsAtom> doctypeName = NS_Atomize(qName);
@@ -837,13 +824,11 @@ nsresult txMozillaXMLOutput::createResultDocument(const nsAString& aName,
       }
 
       // Indicate that there is no internal subset (not just an empty one)
-      rv = NS_NewDOMDocumentType(getter_AddRefs(documentType), mNodeInfoManager,
-                                 doctypeName, mOutputFormat.mPublicId,
-                                 mOutputFormat.mSystemId, VoidString());
-      NS_ENSURE_SUCCESS(rv, rv);
+      RefPtr<DocumentType> documentType = NS_NewDOMDocumentType(
+          mNodeInfoManager, doctypeName, mOutputFormat.mPublicId,
+          mOutputFormat.mSystemId, VoidString());
 
-      nsCOMPtr<nsIContent> docType = do_QueryInterface(documentType);
-      rv = mDocument->AppendChildTo(docType, true);
+      rv = mDocument->AppendChildTo(documentType, true);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -873,7 +858,7 @@ nsresult txMozillaXMLOutput::createHTMLElement(nsAtom* aName,
 txTransformNotifier::txTransformNotifier()
     : mPendingStylesheetCount(0), mInTransform(false) {}
 
-txTransformNotifier::~txTransformNotifier() {}
+txTransformNotifier::~txTransformNotifier() = default;
 
 NS_IMPL_ISUPPORTS(txTransformNotifier, nsIScriptLoaderObserver,
                   nsICSSLoaderObserver)
@@ -902,7 +887,7 @@ txTransformNotifier::ScriptEvaluated(nsresult aResult,
 }
 
 NS_IMETHODIMP
-txTransformNotifier::StyleSheetLoaded(StyleSheet* aSheet, bool aWasAlternate,
+txTransformNotifier::StyleSheetLoaded(StyleSheet* aSheet, bool aWasDeferred,
                                       nsresult aStatus) {
   if (mPendingStylesheetCount == 0) {
     // We weren't waiting on this stylesheet anyway.  This can happen if
@@ -912,7 +897,7 @@ txTransformNotifier::StyleSheetLoaded(StyleSheet* aSheet, bool aWasAlternate,
   }
 
   // We're never waiting for alternate stylesheets
-  if (!aWasAlternate) {
+  if (!aWasDeferred) {
     --mPendingStylesheetCount;
     SignalTransformEnd();
   }
@@ -938,7 +923,7 @@ void txTransformNotifier::OnTransformEnd(nsresult aResult) {
 
 void txTransformNotifier::OnTransformStart() { mInTransform = true; }
 
-nsresult txTransformNotifier::SetOutputDocument(nsIDocument* aDocument) {
+nsresult txTransformNotifier::SetOutputDocument(Document* aDocument) {
   mDocument = aDocument;
 
   // Notify the contentsink that the document is created
@@ -963,6 +948,7 @@ void txTransformNotifier::SignalTransformEnd(nsresult aResult) {
   nsCOMPtr<nsIScriptLoaderObserver> kungFuDeathGrip(this);
 
   if (mDocument) {
+    mDocument->ScriptLoader()->DeferCheckpointReached();
     mDocument->ScriptLoader()->RemoveObserver(this);
     // XXX Maybe we want to cancel script loads if NS_FAILED(rv)?
 
